// BV4242 is a touch keypad with LCD front panel
// This helps to tune the EERPOM settings
//
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <wiringPi.h>
#include <wiringPiI2C.h>
#include "bv4612.h"

// just part of this file
#include "sv3_i2c.c"

// keypad commands
#define PAD_CLEARBUF    1
#define PAD_KEYSINBUF   2
#define PAD_GETKEY      3
#define PAD_ISINBUF     4
#define PAD_SCANCODE    5
#define PAD_LCDRESET    7
#define PAD_GETAVG      10
#define PAD_GETDELTA    11
#define PAD_RESETEEPROM 20
#define PAD_SLEEP       21
#define LCD_RESET       30
#define LCD_CMD         31
#define LCD_DATA        32
#define LCD_SIGN        35
#define LCD_BACKLIGHT   36
#define LCD_CONTRAST    37
#define LCD_SENDBYTES   38
#define LCD_SETFONT     40
#define LCD_HOMECLR     41
#define LCD_COLUMN      42
#define LCD_PAGE        43
#define LCD_SCROLL      44
#define LCD_WRITE       45

// EEPROM constants
#define EE_TRIGGERH 18
#define EE_TRIGGERL EE_TRIGGERH+1
#define EE_HYST 20
#define EE_KEYPTR 21
#define EE_KEYSIZE EE_KEYPTR+1
#define EE_DEBOUNCE 23
#define EE_REPEATH 24
#define EE_REPEATL EE_REPEATH+1
#define EE_TIMEBASE 26
#define EE_BL 27

// *****************************************************************************
// gets a 16 bit value
// *****************************************************************************
void get16(int bus, uint16_t *values, uint8_t cmd)
{
uint16_t v, j;
    wiringPiI2CWrite(bus,cmd); // send command
    for(j=0; j<8; j++) {
        v = wiringPiI2CRead(bus) << 8;
        v += wiringPiI2CRead(bus);
        *(values+j) = v;
    }
}

// =============================================================================
// =============================================================================
int bv4612_join(uint8_t i2caddress)
{
    return(wiringPiI2CSetup(i2caddress));
}

// =============================================================================
// =============================================================================
// =============================================================================
// Keypad section
// =============================================================================
// *****************************************************************************
// Clear keypad buffer
// *****************************************************************************
void clrBuf(int bus)
{
    wiringPiI2CWrite(bus,PAD_CLEARBUF);
    delayMicroseconds(250);
}

// *****************************************************************************
// Gets number of keys in buffer
// *****************************************************************************
uint8_t keysBuf(int bus)
{
    wiringPiI2CWrite(bus,PAD_KEYSINBUF);
    return wiringPiI2CRead(bus);
}

// *****************************************************************************
// Gets a key returns 0 if no keys in buffer
// *****************************************************************************
uint8_t key(int bus)
{
    wiringPiI2CWrite(bus,PAD_GETKEY);
    return wiringPiI2CRead(bus);
//    return wiringPiI2CReadReg8(bus,PAD_GETKEY);
}

// *****************************************************************************
// See if a partiular key is in the buffer, returns 0 if not and it it is
// the position of the key within the buffer
// *****************************************************************************
uint8_t keyIn(int bus, uint8_t k)
{
    wiringPiI2CWriteReg8(bus,PAD_ISINBUF,k);
    return wiringPiI2CRead(bus);
}

// *****************************************************************************
// returns a key scan code 0 of no key pressed
// *****************************************************************************
uint8_t scan(int bus)
{
    wiringPiI2CWrite(bus,PAD_SCANCODE);
    return wiringPiI2CRead(bus);
}

// *****************************************************************************
// sets back light brightness 0 - 130 but setting higher does not matter
// *****************************************************************************
void bl(int bus, uint8_t v)
{
    wiringPiI2CWriteReg8(bus,PAD_ISINBUF,v);
}

// *****************************************************************************
// returns 8 average values in the buffer provided, values are 16 bits
// *****************************************************************************
void average(int bus, uint16_t *b)
{
    get16(bus , b, PAD_GETAVG);
}

// *****************************************************************************
// returns 8 delta values in the buffer provided, values are 16 bits
// *****************************************************************************
void delta(int bus, uint16_t *b)
{
    get16(bus , b, PAD_GETDELTA);
}

// *****************************************************************************
// Resets eeprom values
// *****************************************************************************
void EEreset(int bus)
{
    wiringPiI2CWrite(bus,PAD_RESETEEPROM); // command
}

// *****************************************************************************
// Sleep mode, the keypad will not function in this mode but the device can 
// be awakend vie a read or write to I2C
// *****************************************************************************
void sleep(int bus)
{
    wiringPiI2CWrite(bus,PAD_SLEEP); // command
}

// =============================================================================
// SV3 system section
// =============================================================================
// *****************************************************************************
// Writes a byte to the eeprom at a given address
// *****************************************************************************
void EEWrite(int bus, uint8_t adr, uint8_t value)
{
    sv3_eewrite(bus,adr,value);
}

// *****************************************************************************
// Reads a byte from eeprom at given address
// *****************************************************************************
uint8_t EEread(int bus, uint8_t adr)
{
    return sv3_eeread(bus,adr);
}

// *****************************************************************************
// Get device ID as a 16 bit number
// *****************************************************************************
uint16_t ID(int bus)
{
    return sv3_deviceID(bus);
}

// *****************************************************************************
// Get firmware version as a string, 4 bytes needed
// *****************************************************************************
void Version(int bus, char *s)
{
    sv3_Version(bus,s);
}

// =============================================================================
// EEPROM settings
// =============================================================================
// *****************************************************************************
// sets new trigger value
// *****************************************************************************
void trigger(int bus, uint16_t value)
{
    EEWrite(bus,EE_TRIGGERH,(value >> 8) & 0xff);
    EEWrite(bus,EE_TRIGGERL,value & 0xff);
}

// *****************************************************************************
// sets new hysteresis value
// *****************************************************************************
void hyst(int bus, uint8_t value)
{
    EEWrite(bus, EE_HYST,value);
}

// *****************************************************************************
// sets new keytable pointer
// *****************************************************************************
void keyPtr(int bus, uint8_t value)
{
    EEWrite(bus, EE_KEYPTR,value);
}

// *****************************************************************************
// sets new key table size
// *****************************************************************************
void keySize(int bus, uint8_t value)
{
    EEWrite(bus, EE_KEYSIZE,value);
}

// *****************************************************************************
// sets new debounce
// *****************************************************************************
void debounce(int bus, uint8_t value)
{
    EEWrite(bus, EE_DEBOUNCE,value);
}

// *****************************************************************************
// sets new repeat value
// *****************************************************************************
void repeat(int bus, uint16_t value)
{
    EEWrite(bus,EE_REPEATH,(value >> 8) & 0xff);
    EEWrite(bus,EE_REPEATL,value & 0xff);
}

// *****************************************************************************
// sets new timebase
// *****************************************************************************
void timebase(int bus, uint8_t value)
{
    EEWrite(bus,EE_TIMEBASE,value);
}

// *****************************************************************************
// sets new defailt bl
// *****************************************************************************
void defaultBL(int bus,uint8_t value)
{
    EEWrite(bus,EE_BL,value);
}

// =============================================================================
// =============================================================================
// =============================================================================
// LCD Section
// This is for the ST7032i controller and follows the same commands used 
// for LiquidCrystal; to this end column is specified first i.e 16x2
// =============================================================================
// 
// *****************************************************************************
// sends a command to the display, bus open at this point
// *****************************************************************************
void lcd_reset(int bus)
{
    wiringPiI2CWrite(bus,LCD_RESET);
    delay(350); // commands takes 300mS
}

// *****************************************************************************
// sends a command to the display, bus open at this point
// *****************************************************************************
void lcd_cmd(int bus, uint8_t cmd)
{
    wiringPiI2CWriteReg8(bus,LCD_CMD,cmd);
}

// *****************************************************************************
// sends data to the display, bus open at this point
// *****************************************************************************
void lcd_write(int bus, uint8_t data)
{
    wiringPiI2CWriteReg8(bus,LCD_WRITE,data);
}

// *****************************************************************************
// *****************************************************************************
void lcd_string(int bus, int row, int col, char *s)
{
int del=strlen(s)*5; // delay per char
    lcd_setCursor(bus,col,row);
    while(*s) {
        lcd_write(bus,*(s++));
    }
    delay(del);
}


// *****************************************************************************
//  
// *****************************************************************************
void lcd_contrast(int bus,uint8_t v)
{
    wiringPiI2CWriteReg8(bus,LCD_CONTRAST,v); 
}

// *****************************************************************************
//  
// *****************************************************************************
void lcd_bl(int bus,uint8_t v)
{
    wiringPiI2CWriteReg8(bus,LCD_BACKLIGHT,v); 
}

// *****************************************************************************
// Sets column address
// *****************************************************************************
void lcd_column(int bus,uint8_t v)
{
    wiringPiI2CWriteReg8(bus,LCD_COLUMN,v);
}

// *****************************************************************************
// Sets row address
// *****************************************************************************
void lcd_page(int bus,uint8_t v)
{
    wiringPiI2CWriteReg8(bus,LCD_PAGE,v);
}

// *****************************************************************************
//
// *****************************************************************************
void lcd_clear(int bus)
{
    wiringPiI2CWrite(bus,LCD_HOMECLR);
    delay(350); // commands takes 300mS
}

// *****************************************************************************
// *****************************************************************************
void lcd_home(int bus)
{
    lcd_column(bus,0);
    lcd_page(bus,0);
}

// *****************************************************************************
// *****************************************************************************
void lcd_setCursor(int bus,uint8_t col, uint8_t row)
{
uint8_t adr;
    if(row > 7) row = 7;
    if(col > 127) col = 127;
    lcd_column(bus,col);
    lcd_page(bus,row);
}

// int main(int argc, char *argv[])
// {
//   int pad=bv4612_join(0x35); // attache to bus
//   lcd_clear(pad);
//   lcd_string(pad,3,3,"hello fred");
// }

